package org.open.util;

import java.util.Base64;
import java.util.Map;

import javax.crypto.Cipher;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;

import org.apache.commons.lang3.StringUtils;

import com.alibaba.fastjson.JSON;

public class AscSecurityHelper {
    /*
     * 加密用的Key 可以用26个字母和数字组成 使用AES-128-CBC加密模式，key需要为16位。
     * 
     */
    private static final String iv = "Qing&95GUY86Hong";
    
    /**
     * @Description AES算法加密明文
     * @param param 明文Map
     * @param appSecret 密钥
     * @return 密文
     */
    public static String encryptAES(String appSecret, Map<String, Object> params) {
        String stringSignTemp = JSON.toJSONString(params);
        return encryptAES(appSecret, stringSignTemp);
    }

    /**
     * @Description AES算法加密明文
     * @param data 明文
     * @param appSecret 密钥
     * @return 密文
     */
    public static String encryptAES(String appSecret, String data) {
        if(StringUtils.isNotBlank(appSecret) && appSecret.length()>16) {
            appSecret = appSecret.substring(0, 16);
        } else {
            return null;
        }
        try {
            Cipher cipher = Cipher.getInstance("AES/CBC/NoPadding");
            int blockSize = cipher.getBlockSize();
            byte[] dataBytes = data.getBytes();
            int plaintextLength = dataBytes.length;
            if (plaintextLength % blockSize != 0) {
                plaintextLength = plaintextLength + (blockSize - (plaintextLength % blockSize));
            }
            byte[] plaintext = new byte[plaintextLength];
            System.arraycopy(dataBytes, 0, plaintext, 0, dataBytes.length);
            SecretKeySpec keyspec = new SecretKeySpec(appSecret.getBytes(), "AES");
            IvParameterSpec ivspec = new IvParameterSpec(iv.getBytes()); // CBC模式，需要一个向量iv，可增加加密算法的强度
            cipher.init(Cipher.ENCRYPT_MODE, keyspec, ivspec);
            byte[] encrypted = cipher.doFinal(plaintext);
            return encode(encrypted).trim(); // BASE64做转码。
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    /**
     * @Description AES算法解密密文
     * @param data 密文
     * @param appSecret  密钥
     * @return 明文
     */
    public static String decryptAES(String appSecret, String data) {
        if(StringUtils.isNotBlank(appSecret) && appSecret.length()>16) {
            appSecret = appSecret.substring(0, 16);
        } else {
            return null;
        }
        try {
            byte[] encrypted1 = decode(data);// 先用base64解密
            Cipher cipher = Cipher.getInstance("AES/CBC/NoPadding");
            SecretKeySpec keyspec = new SecretKeySpec(appSecret.getBytes(), "AES");
            IvParameterSpec ivspec = new IvParameterSpec(iv.getBytes());
            cipher.init(Cipher.DECRYPT_MODE, keyspec, ivspec);
            byte[] original = cipher.doFinal(encrypted1);
            String originalString = new String(original);
            return originalString.trim();
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    /**
     * 编码
     * @param byteArray
     * @return
     */
    private static String encode(byte[] byteArray) {
        return Base64.getEncoder().encodeToString(byteArray);
    }

    /**
     * 解码
     * @param base64EncodedString
     * @return
     */
    private static byte[] decode(String base64EncodedString) {
        return Base64.getDecoder().decode(base64EncodedString);
    }
    
    public static void main(String[] args) throws Exception {
        String data = "{data: 'hong123456'}";
        String appSecret = "9e839853592f45f7baaf138ed6a5c631";
        String tmp = AscSecurityHelper.encryptAES(appSecret, data);
        System.out.println(tmp);
        String tmp2 = AscSecurityHelper.decryptAES(appSecret, tmp);
        System.out.println(tmp2);
    }
}
