package com.qing.kdniao.action.web;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.jeespring.common.web.AbstractBaseController;
import org.open.util.AscSecurityHelper;
import org.open.util.HttpUtil;
import org.open.util.SignUtil;
import org.open.util.AscSecurityHelper;

/**
 * 快递服务模块 单点登录服务
 */
@Controller
@RequestMapping(value = "/rest/parcel")
public class PortalRestController extends AbstractBaseController {
	public final static String SESSION_PARCEL_CODE = "session_parcel_code";
	
	//晋来办服务域名地址
	//测试环境地址：http://bmfwtest.jcgov.gov.cn
	//生产环境地址：https://bmfw.jcgov.gov.cn
	private String domainAddress = "http://bmfwtest.jcgov.gov.cn";
			
	/**
	 * 单点登录服务
	 * @param token 授权令牌
	 * @param code 上架服务设置的服务code
	 * @param pageurl 个人消息跳转地址
	 */
	@RequestMapping("portal")
	public String portal(HttpServletRequest request, HttpServletResponse response, 
			String token, String code, String pageurl) throws Exception {
        logger.info("<进入> 快递服务单点登录");
        logger.info("token：{}", token);
        if (StringUtils.isBlank(token)) {
            writeResponse(response, "token为空");
            return null;
        }
        if (StringUtils.isBlank(code)) {
            writeResponse(response, "服务code为空");
            return null;
        }
		
		//分配CODE和秘钥，申报应用审核通过后获得
        String appCode = "cdd9****************************";
        String appSecret = "5113****************************";
        //上架服务模块
        String moduleCode = "6*******************************"; 
        
        //通过回调地址接收令牌
        //令牌：为一次性令牌，使用后需要从晋来办重新获取，未使用的情况下2分钟作废
        String authtoken = "111111111111111111111111111111111111"; // 注：（仅在测试环境 联调时有效)可以重复获取
        
        String timestamp = SignUtil.getTimeStamp();
        String nonce = SignUtil.getNonce();
        
        Map<String, Object> param = new HashMap<String, Object>();
        param.put("appcode", appCode);
        param.put("modulecode", moduleCode);
        param.put("authtoken", authtoken);
        param.put("timestamp", timestamp);
        param.put("nonce", nonce);
        
        String sign = SignUtil.createSign(appSecret, param); //生成数字签名
        param.put("sign", sign);
        String data = AscSecurityHelper.encryptAES(appSecret, param); //生成加密信息
        
        Map<String, Object> postData = new HashMap<String, Object>();
        postData.put("appcode", appCode);
        postData.put("data", data);
        System.out.println("appcode="+ appCode);
        System.out.println("data="+ data);
        String url = domainOrIpAddress+ "/rest/authuser/getUserInfoNew";
        String result = HttpUtil.sendPost(url, postData);
        System.out.println("返回信息："+result);
        
        if(!getJSONType(result)) {
			writeResponse(response, "服务连接异常");
			return null;
        }
        
        JSONObject obj = JSONObject.parseObject(result);
        String resp_state = obj.getString("state");
        String resp_msg = obj.getString("msg");
        String resp_data = obj.getString("data");
        
        if("200".equals(resp_state)) {
            String decryptData = AscSecurityHelper.decryptAES(appSecret, resp_data); //解密信息
            System.out.println("解密信息：" + decryptData);
            
            JSONObject res_obj = JSONObject.parseObject(decryptData);
            //验签
            Map reqMap = JSON.parseObject(res_obj.toJSONString(), Map.class);
            boolean flag = checkSign(appSecret, reqMap);
            if(flag) {
                String res_userInfo = res_obj.getString("userInfo");
                JSONObject userInfo_obj = JSONObject.parseObject(res_userInfo);
                System.out.println("用户信息:" + userInfo_obj.toJSONString());
				
				String userId = userInfo_obj.getString("id"); //用户ID
				String loginName = userInfo_obj.getString("loginName"); //登录用户手机号
				//业务系统 判断用户是否已经注册
				//步骤一.未注册情况：实现用户注册
				//步骤二,已注册情况：保存用户session
				HttpSession session = request.getSession();
				//根据业务需求绑定session信息
				session.setAttribute(SESSION_PARCEL_CODE, userId); 
				//根据传入CODE跳转至对应的服务模块
            } else {
				writeResponse(response, "验签错误");
				return null;
            }
        } else {
			writeResponse(response, resp_msg);
			return null;
        }
	}
	
	private void writeResponse(HttpServletResponse response, String responseString) throws IOException {
		String tmp = String.valueOf(responseString.getBytes().length);
		response.setHeader("Content-Length", tmp);
		ServletOutputStream out = response.getOutputStream();
		out.write(responseString.getBytes());
		out.flush();
		out.close();
	}
}
